
local function ActionButton(inst)

	local action_target = GLOBAL.FindEntity(inst, 6, function(guy) return (guy.components.edible and inst.components.eater:CanEat(guy)) or
		 													 (guy.components.workable and guy.components.workable.workable and inst.components.worker:CanDoAction(guy.components.workable.action)) end)

	if not inst.sg:HasStateTag("busy") and action_target then
		if (action_target.components.edible and inst.components.eater:CanEat(action_target)) then
			return GLOBAL.BufferedAction(inst, action_target, GLOBAL.ACTIONS.EAT)
		elseif action_target.components.workable.workable then
			return GLOBAL.BufferedAction(inst, action_target, action_target.components.workable.action)
		end
	end
end

local function LeftClickPicker(inst, target_ent, pos)
    if inst.components.combat:CanTarget(target_ent) then
        return inst.components.playeractionpicker:SortActionList({GLOBAL.ACTIONS.ATTACK}, target_ent, nil)
    end

	if target_ent and target_ent.components.edible and inst.components.eater:CanEat(target_ent) then
		return inst.components.playeractionpicker:SortActionList({GLOBAL.ACTIONS.EAT}, target_ent, nil)
	end

    if target_ent and target_ent.components.workable and target_ent.components.workable.workable and inst.components.worker:CanDoAction(target_ent.components.workable.action) then
        return inst.components.playeractionpicker:SortActionList({target_ent.components.workable.action}, target_ent, nil)
    end
end

local function RightClickPicker(inst, target_ent, pos)
	return {}
end

local function EnableActionButton()
	if not GetPlayer().components.worker then
		GetPlayer():AddComponent("worker")
	end
	GetPlayer().components.playercontroller.actionbuttonoverride = ActionButton
	GetPlayer().components.playeractionpicker.leftclickoverride = LeftClickPicker
	GetPlayer().components.playeractionpicker.rightclickoverride = RightClickPicker
end

local function SetBank(bank) GetPlayer().AnimState:SetBank(bank) end
local function SetBuild(build) GetPlayer().AnimState:SetBuild(build) end
local function Damage(damage) GetPlayer().components.combat:SetDefaultDamage(damage) end
local function ResetScale() GetPlayer().Transform:SetScale(1,1,1) end
local function Health(health) GetPlayer().components.health:SetMaxHealth(health) end
local function ResetCombat() GetPlayer().components.combat:SetRange(3) GetPlayer().components.combat:SetAreaDamage(nil) end
local function RunSpeed(runspeed) GetPlayer().components.locomotor.runspeed = runspeed end
local function WalkSpeed(walkspeed) GetPlayer().components.locomotor.walkspeed = walkspeed end

function Transform(mob)
	EnableActionButton()
	GetPlayer().components.eater.monsterimmune = true
	GetPlayer().components.sanity.dapperness = TUNING.DAPPERNESS_HUGE
	GetPlayer().components.talker:IgnoreAll()
	if not GetPlayer().components.follower and not GetPlayer().components.lootdropper then
		GetPlayer():AddComponent("follower")
		GetPlayer():AddComponent("lootdropper")
	end

	if mob == "beefalo" then
		Health(500)
		Damage(34)
		SetBank(mob)
		SetBuild(mob .. "_build")
		RunSpeed(7)
		GetPlayer().sounds =
		{
			walk = "dontstarve/beefalo/walk",
			grunt = "dontstarve/beefalo/grunt",
			yell = "dontstarve/beefalo/yell",
			swish = "dontstarve/beefalo/tail_swish",
			curious = "dontstarve/beefalo/curious",
			angry = "dontstarve/beefalo/angry",
		}
		GetPlayer().components.locomotor.triggerscreep = true
		GetPlayer().entity:AddDynamicShadow():SetSize( 6, 2 )
		GetPlayer():SetStateGraph("SGBeefalo")
		ResetCombat()
		GetPlayer():RemoveTag("spider")
		GetPlayer():RemoveTag("monster")
	end
	if mob == "minotaur" then
		Health(2500)
		Damage(100)
		RunSpeed(TUNING.MINOTAUR_RUN_SPEED)
		SetBank("rook")
		SetBuild("rook_rhino")
		GetPlayer():SetStateGraph("SGminotaur")
		GetPlayer().entity:AddDynamicShadow():SetSize( 5, 3)
	end
	if mob == "manrabbit" then
		Health(200)
		Damage(40)
		RunSpeed(TUNING.PIG_RUN_SPEED)
		SetBank(mob)
		SetBuild(mob .. "_build")
		GetPlayer():AddTag("pig")
		GetPlayer().entity:AddDynamicShadow():SetSize( 1.5, .75 )
		GetPlayer():SetStateGraph("SGbunnyman")
		GetPlayer().Transform:SetScale(1.25, 1.25, 1.25)
		GetPlayer():RemoveTag("beefalo")
	end
	if mob == "manrabbit_beard" then
		Damage(TUNING.BEARDLORD_DAMAGE)
		SetBuild(mob .. "_build")
	end
	if mob == "hound" then
		Health(TUNING.HOUND_HEALTH)
		Damage(TUNING.HOUND_DAMAGE)
		SetBank(mob)
		SetBuild(mob)
		RunSpeed(TUNING.HOUND_SPEED)
		GetPlayer():AddTag("hound")
		GetPlayer():AddTag("monster")
		GetPlayer().entity:AddDynamicShadow():SetSize( 2.5, 1.5 )
		GetPlayer():SetStateGraph("SGhound")
		GetPlayer():RemoveTag("manrabbit")
		ResetScale()
	end
	if mob == "hound_ice" then
		Health(TUNING.ICEHOUND_HEALTH)
		Damage(TUNING.ICEHOUND_DAMAGE)
		SetBuild(mob)
	end
	if mob == "hound_red" then
		Health(TUNING.FIREHOUND_HEALTH)
		Damage(TUNING.FIREHOUND_DAMAGE)
		SetBuild(mob)
	end
	if mob == "deerclops" then
		Health(2000)
		Damage(150)
		WalkSpeed(3)
		SetBank(mob)
		SetBuild(mob .. "_build")
		GetPlayer().entity:AddDynamicShadow():SetSize( 6, 3.5 )
		GetPlayer():SetStateGraph("SGdeerclops")
		GetPlayer().components.combat:SetRange(8)
		GetPlayer().components.combat:SetAreaDamage(6, 0.8)
		GetPlayer().Transform:SetScale(1.65, 1.65, 1.65)
		GetPlayer():RemoveTag("hound")
	end
	if mob == "leif" then
		Health(TUNING.LEIF_HEALTH)
		Damage(TUNING.LEIF_DAMAGE)
		WalkSpeed(1.5 * 2)
		SetBank(mob)
		SetBuild(mob .. "_build")
		GetPlayer():AddTag("leif")
		GetPlayer():AddTag("tree")
		GetPlayer().entity:AddDynamicShadow():SetSize( 4, 1.5 )
		GetPlayer():SetStateGraph("SGLeif")
		ResetScale()
	end
	if mob == "pigman" then
		Health(250)
		Damage(33)
		RunSpeed(TUNING.PIG_RUN_SPEED)
		SetBank("pigman")
		SetBuild("pig_build")
		GetPlayer():AddTag("pig")
		GetPlayer().entity:AddDynamicShadow():SetSize( 1.5, .75 )
		GetPlayer():SetStateGraph("SGpig")
		GetPlayer():RemoveTag("leif")
		GetPlayer():RemoveTag("tree")
		GetPlayer():RemoveTag("monster")
		ResetCombat()
	end
	if mob == "pig_guard" then
		Health(300)
		SetBuild(mob .. "_build")
	end
	if mob == "werepig" then
		Health(350)
		Damage(40)
		SetBuild(mob .. "_build")
		GetPlayer():SetStateGraph("SGwerepig")
	end
	if mob == "spider" then
		Health(100)
		Damage(20)
		RunSpeed(TUNING.SPIDER_RUN_SPEED)
		SetBank(mob)
		SetBuild(mob .. "_build")
		GetPlayer():AddTag("spider")
		GetPlayer():AddTag("monster")
		GetPlayer().components.locomotor.triggerscreep = false
		GetPlayer().entity:AddDynamicShadow():SetSize( 1.5, .5 )
		GetPlayer():SetStateGraph("SGspider")
	end
	if mob == "spider_warrior" then
		Health(200)
		RunSpeed(TUNING.SPIDER_WARRIOR_RUN_SPEED)
		SetBuild(mob .. "_build")
		GetPlayer():AddTag("spider_warrior")
		GetPlayer().components.combat:SetRange(TUNING.SPIDER_WARRIOR_ATTACK_RANGE, TUNING.SPIDER_WARRIOR_HIT_RANGE)
	end
	if mob == "spiderqueen" then
		Health(1250)
		Damage(80)
		WalkSpeed(TUNING.SPIDERQUEEN_WALKSPEED * 2)
		SetBank("spider_queen")
		SetBuild("spider_queen_build")
		GetPlayer():SetStateGraph("SGspiderqueen")
		GetPlayer().components.combat:SetRange(5)
		GetPlayer().entity:AddDynamicShadow():SetSize( 7, 3 )
		GetPlayer():RemoveTag("spider_warrior")
	end
	if mob == "worm" then
		Health(900)
		Damage(75)
		RunSpeed(4)
		SetBank(mob)
		SetBuild(mob)
		GetPlayer():SetStateGraph("SGworm")
		GetPlayer().entity:AddDynamicShadow():SetSize( 1.5, .5 )
	end

	----ROG
	if GLOBAL.IsDLCEnabled(GLOBAL.REIGN_OF_GIANTS) then
		--[[
		if mob == "dragonfly" then
			Health(2750)
			SetBank(mob)
			SetBuild(mob .. "_build")
			GetPlayer():SetStateGraph("SGdragonfly")
			GetPlayer().components.eater:AbleToEat("ash")
			GetPlayer():AddComponent("propagator")
			GetPlayer():AddComponent("groundpounder")
			GetPlayer().components.groundpounder.numRings = 2
			GetPlayer().components.groundpounder.burner = true
			GetPlayer().components.groundpounder.groundpoundfx = "firesplash_fx"
			GetPlayer().components.groundpounder.groundpounddamagemult = .5
			GetPlayer().components.groundpounder.groundpoundringfx = "firering_fx"
			GetPlayer().Transform:SetScale(1.3, 1.3, 1.3)
		end
		--]]
	end
end


